"""Constants for the emulated_roku component."""
DOMAIN = "emulated_roku"

CONF_SERVERS = "servers"
CONF_LISTEN_PORT = "listen_port"
CONF_HOST_IP = "host_ip"
CONF_ADVERTISE_IP = "advertise_ip"
CONF_ADVERTISE_PORT = "advertise_port"
CONF_UPNP_BIND_MULTICAST = "upnp_bind_multicast"

DEFAULT_NAME = "Home Assistant"
DEFAULT_PORT = 8060
