"""Constants for the Co2signal integration."""


DOMAIN = "co2signal"
CONF_COUNTRY_CODE = "country_code"
ATTRIBUTION = "Data provided by CO2signal"
MSG_LOCATION = (
    "Please use either coordinates or the country code. "
    "For the coordinates, "
    "you need to use both latitude and longitude."
)
