"""Constants for the SUPLA MQTT integration."""

DOMAIN = "ais_supla_mqtt"
OAUTH_URL = "https://cloud.supla.org/oauth/v2/auth"
TOKEN_URL = "https://cloud.supla.org/oauth/v2/token"
REDIRECT_URL = "https://AIS_HOST/ords/dom/auth/supla_mqtt_bridge_callback"
AUTH_SCOPE = "mqtt_broker"
API_ENDPOINT = "/api/integrations/mqtt-credentials"
